/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ai;

import ai.t.AIClientT;
import ai.t.RandomAI;
import ai.t.SwarmAIT;
import client.network.ServerHandler;
import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPException;
import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.Switch;
import common.hive.PlayerColor;
import java.net.InetAddress;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import model.ds.GameBoard;

/**
 *
 * @author gross
 */
public class RandomMain {

    public static void main(String args[]) throws JSAPException {
        InetAddress server;
        int port;
        String login;
        String password;
        List<String> buffer = new LinkedList<String>();
        if (args != null && args.length > 0 && !args[0].startsWith("-")) {
            if (args.length == 4) {
                buffer.add("-s");
                buffer.add(args[0]);
                buffer.add("-p");
                buffer.add(args[1]);
                buffer.add("-g");
                buffer.add(args[2]);
                buffer.add("-c");
                buffer.add(args[3]);
            } else if (args.length == 7) {
                buffer.add("-s");
                buffer.add(args[0]);
                buffer.add("-p");
                buffer.add(args[1]);
                buffer.add("-l");
                buffer.add(args[2]);
                buffer.add("-x");
                buffer.add("\"" + args[3] + " " + args[4] + "\"");
                buffer.add("-g");
                buffer.add(args[5]);
                buffer.add("-c");
                buffer.add(args[6]);
            }
            args = buffer.toArray(new String[0]);
        }

        JSAP jsap = new JSAP();

        FlaggedOption optServer = new FlaggedOption("server").setStringParser(JSAP.INETADDRESS_PARSER).setDefault("127.0.0.1").setRequired(false).setShortFlag('s').setLongFlag("server");
        optServer.setHelp("The address of the hive server.");

        FlaggedOption optPort = new FlaggedOption("port").setStringParser(JSAP.INTEGER_PARSER).setDefault("25052").setRequired(false).setShortFlag('p').setLongFlag("port");
        optPort.setHelp("The port of the server.");

        FlaggedOption optUser = new FlaggedOption("login").setStringParser(JSAP.STRING_PARSER).setDefault("").setRequired(false).setShortFlag('l').setLongFlag("login");
        optUser.setHelp("The user name to login on the server. If the name contains spaces, the name has to be in quotation marks. An empty (or not provided) user name is used for anonymous login.");

        FlaggedOption optPassword = new FlaggedOption("password").setStringParser(JSAP.STRING_PARSER).setDefault("").setRequired(false).setShortFlag('x').setLongFlag("password");
        optPassword.setHelp("The clients password. If the login is not anonymous, a pasword has to be given.");

        FlaggedOption optGame = new FlaggedOption("game").setStringParser(JSAP.STRING_PARSER).setRequired(false).setShortFlag('g').setLongFlag("game");
        optGame.setHelp("The name of the game in which the AI should participate in.");

        FlaggedOption optColor = new FlaggedOption("color").setStringParser(JSAP.STRING_PARSER).setDefault("w").setRequired(false).setShortFlag('c').setLongFlag("color");
        optColor.setHelp("The color which the AI should play. b (=black) or w (=white).");

        Switch optHelp = new Switch("help").setDefault("false").setShortFlag('h').setLongFlag("help");
        optHelp.setHelp("Shows this help.");

        jsap.registerParameter(optServer);
        jsap.registerParameter(optPort);
        jsap.registerParameter(optUser);
        jsap.registerParameter(optPassword);
        jsap.registerParameter(optGame);
        jsap.registerParameter(optColor);
        jsap.registerParameter(optHelp);

        JSAPResult config = jsap.parse(args);

        Iterator<?> i = config.getErrorMessageIterator(); // List of strings
        while (i.hasNext()) {
            System.err.println(i.next().toString());
        }

        if (!config.success()) {
            System.out.println(jsap.getHelp());
        } else {
            login = config.getString("login");
            server = config.getInetAddress("server");
            port = config.getInt("port");
            password = config.getString("password");

            String color = config.getString("color").toLowerCase();
            if (!(color.equals("w") || color.equals("b"))) {
                System.err.println("Player color '" + color + "' not known.");
                System.out.println(jsap.getHelp());
                System.exit(1);
            }
            if (!config.contains("game")) {
                System.err.println("No game specified for the AI.");
                System.out.println(jsap.getHelp());
                System.exit(1);
            }
            ai(server, port, login, password, config.getString("game"), color.equals("w") ? PlayerColor.WHITE : PlayerColor.BLACK);

        }
    }

    private static void ai(InetAddress server, int port, String login, String password, String game, PlayerColor color) {
        final HiveAI<?, ?> swarmAI = new SwarmAIT(login, password, color, game) {
            @Override
            protected AIClientT createAI() {
                gameBoard = new GameBoard();
                ai = new RandomAI((GameBoard) gameBoard, playerColor, -1, -1);
                return (RandomAI) ai;
            }
        };
        ServerHandler handler = new ServerHandler(Integer.toString(port), server.getCanonicalHostName(), swarmAI, false);
        swarmAI.setServerHandler(handler);
        handler.start();
    }
}
